package com.example.flutter_icon_network;

import java.io.IOException;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;
import java.util.List;

import foundation.icon.icx.IconService;
import foundation.icon.icx.Request;
import foundation.icon.icx.data.Block;
import foundation.icon.icx.data.Bytes;
import foundation.icon.icx.data.ConfirmedTransaction;
import foundation.icon.icx.data.ScoreApi;
import foundation.icon.icx.transport.http.HttpProvider;
import okhttp3.OkHttpClient;

class ICONSyncBlockManager {
    private static ICONSyncBlockManager instance;
    private final IconService iconService;

    private ICONSyncBlockManager(String domain) {
        OkHttpClient okHttpClient = new OkHttpClient.Builder()
                .readTimeout(2000, TimeUnit.MILLISECONDS)
                .writeTimeout(6000, TimeUnit.MILLISECONDS)
                .build();
        iconService = new IconService(new HttpProvider(okHttpClient, domain));
    }

    public static ICONSyncBlockManager getInstance(String domain) {
        if (instance == null) {
            instance = new ICONSyncBlockManager(domain);
            return instance;
        }
        return instance;
    }

    public String getLastBlock() throws IOException {
        Block lastBlock = iconService.getLastBlock().execute();
        return lastBlock.toString();
    }

    public List<String> getConfirmedTransactionList() throws IOException {
        Block lastBlock = iconService.getLastBlock().execute();
        List<ConfirmedTransaction> txList = lastBlock.getTransactions();
        List<String> txHashList = new ArrayList<>();

        if (txList != null) {
            for (ConfirmedTransaction transaction : txList) {
                txHashList.add(transaction.getTxHash().toHexString(true));
            }
        }

        return txHashList;
    }

    public Block getBlockByHeight(BigInteger blockHeight) {
        Block block = null;
        try {
            block = iconService.getBlock(blockHeight).execute();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return block;
    }

    public Block getBlockByHash(Bytes hash) {
        Block block = null;
        try {
            block = iconService.getBlock(hash).execute();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return block;
    }

    BigInteger getBlockTotalSupply() {
        BigInteger totalSupply = null;
        try {
            totalSupply = iconService.getTotalSupply().execute();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return totalSupply;
    }
}